Package: ecostats
Title: Code and Data Accompanying the Eco-Stats Text (Warton 2022)
Version: 1.2.2
Date: 2025-06-06
Authors@R: c(person("David", "Warton", email = "david.warton@unsw.edu.au", role = c("aut", "cre")),
            person("Christopher", "Chung", role = "ctb"),
            person("Mark", "Donoghoe", role = "ctb"),
            person("Eve", "Slavich", role = "ctb")
            )
Description: Functions and data supporting the Eco-Stats text (Warton, 2022, Springer), and solutions to exercises. Functions include tools for using simulation envelopes in diagnostic plots, and a function for diagnostic plots of multivariate linear models. Datasets mentioned in the package are included here (where not available elsewhere) and there is a vignette for each chapter of the text with solutions to exercises.
License: LGPL (>= 2.1)
Imports: ecoCopula, GET, graphics, grDevices, MASS, methods, mgcv,
        mvtnorm, parallel, stats
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
Depends: R (>= 3.5.0), mvabund (>= 4.2)
Suggests: knitr, rmarkdown, ade4, caper, car, corrplot, DAAG, DHARMa,
        dplyr, gclus, GGally, ggplot2, ggthemes, glmnet, gllvm,
        glmmTMB, GPArotation, grplasso, lattice, leaps, lme4, MCMCglmm,
        multcomp, nlme, ordinal, permute, pgirmess, phylobase,
        phylosignal, psych, reshape2, smatr, testthat, vegan, VGAM,
        covr
VignetteBuilder: knitr
URL: https://dwarton.github.io/ecostats/
NeedsCompilation: no
Packaged: 2025-06-06 04:48:01 UTC; z3103495
Author: David Warton [aut, cre],
  Christopher Chung [ctb],
  Mark Donoghoe [ctb],
  Eve Slavich [ctb]
Maintainer: David Warton <david.warton@unsw.edu.au>
Repository: CRAN
Date/Publication: 2025-06-06 05:10:02 UTC
Built: R 4.6.0; ; 2026-04-28 17:00:49 UTC; unix
