# Generated by roxygen2: do not edit by hand

S3method(print,anovaPB)
S3method(simulate,manyglm)
S3method(simulate,mlm)
export(BlockBootID)
export(addSmooth)
export(anovaPB)
export(cpredict)
export(cresiduals)
export(plotenvelope)
export(qqenvelope)
import(ecoCopula)
import(graphics)
import(mvabund)
import(stats)
importFrom(MASS,rnegbin)
importFrom(grDevices,adjustcolor)
importFrom(methods,.hasSlot)
importFrom(parallel,clusterApplyLB)
importFrom(parallel,clusterExport)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
