# ecostats <img src="man/figures/ecostats_hex.png" align="right" width="120"/>

> Code + Data Accompanying the Eco-Stats Text

<!-- badges: start -->
<!-- [![ci](https://github.com/eco-stats/ecostats/workflows/ci/badge.svg)](https://github.com/eco-stats/ecostats/actions?query=workflow%3Aci) -->
[![DOI](https://img.shields.io/badge/DOI-10.1007/978&hyphen;3&hyphen;030&hyphen;88443&hyphen;7-orange.svg)](https://doi.org/10.1007/978-3-030-88443-7)
[![License](http://img.shields.io/badge/license-LGPL%20(%3E=%202.1)-brightgreen.svg?style=flat)](http://www.gnu.org/licenses/gpl-2.0.html)
[![CRAN](http://www.r-pkg.org/badges/version/ecostats)](https://CRAN.R-project.org/package=ecostats) 
[![Downloads](http://cranlogs.r-pkg.org/badges/ecostats?color=brightgreen)](https://www.r-pkg.org/pkg/ecostats)
[![R-CMD-check](https://github.com/eco-stats/ecostats/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/eco-stats/ecostats/actions/workflows/R-CMD-check.yaml) 
<!-- [![Codecov test coverage](https://codecov.io/gh/eco-stats/ecostats/branch/ghactions/graph/badge.svg?token=95vH8l02ZK)](https://app.codecov.io/gh/eco-stats/ecostats?branch=master) -->
<!-- badges: end -->

#### Author

David Warton

#### License

LGPL (\>= 2.1)

##### Installation

You can get the latest version of `ecostats` from CRAN:

``` r
install.packages("ecostats")
library(ecostats)
```

But if you want access to this development version, with the latest bells and whistles (where applicable), it can be installed from GitHub using the `devtools` package:

``` r
devtools::install_github('eco-stats/ecostats', ref='main')
library(ecostats)
```
